package uicc.usim.geolocation ;

import uicc.toolkit.* ;

/**
 * This interface defines services allowing an application to get information on the ME current geographical location.
 * Depending on card and terminal capabilities, different means may be used to compute geographical location.
 *
 * <p>see 3GPP TS 23.032 [14] Universal Geographical Area Description (GAD)
 */
public interface GeoLocation {

    /**
     * Horizontal accuracy: best effort
     */
    public static final byte HORIZONTAL_ACCURACY_BEST_EFFORT = (byte)0x81 ;
    /**
     * Vertical coordinate: not requested
     */
    public static final byte VERTICAL_COORDINATE_NOT_REQUESTED = (byte)0x80 ;
    /**
     * Vertical coordinate: best effort
     */
    public static final byte VERTICAL_COORDINATE_REQUESTED_BEST_EFFORT = (byte)0x81 ;
    /**
     * Horizontal velocity requested
     */
    public static final byte HORIZONTAL_VELOCITY_REQUESTED = (byte)0x01 ;
    /**
     * Vertical velocity requested in addition to horizontal velocity
     */
    public static final byte VERTICAL_AND_HORIZONTAL_VELOCITY_REQUESTED = (byte)0x03 ;
    /**
     * Uncertainty of horizontal velocity requested in addition to horizontal velocity
     */
    public static final byte HORIZONTAL_VELOCITY_REQUESTED_WITH_UNCERTAINTY = (byte)0x05 ;
    /**
     * Uncertainty of horizontal and vertical velocity requested in addition to horizontal and vertical velocity
     */
    public static final byte VERTICAL_AND_HORIZONTAL_VELOCITY_REQUESTED_WITH_UNCERTAINTY = (byte)0x0F ;
    /**
     * Horizontal Velocity field is requested from Velocity Information
     */
    public static final byte GET_HORIZONTAL_VELOCITY = (byte)0x00 ;
    /**
     * Vertical Velocity field is requested from Velocity Information
     */
    public static final byte GET_VERTICAL_VELOCITY = (byte)0x01 ;
    /**
     * Horizontal Uncertainty Velocity field is requested from Velocity Information
     */
    public static final byte GET_HORIZONTAL_UNCERTAINTY_VELOCITY = (byte)0x02 ;
    /**
     * Vertical Uncertainty Velocity field is requested from Velocity Information
     */
    public static final byte GET_VERTICAL_UNCERTAINTY_VELOCITY = (byte)0x03 ;
    /**
     * GAD shape: Ellipsoid point
     */
    public static final byte GAD_SHAPES_ELLIPSOID_POINT = (byte)0x01 ;
    /**
     * GAD shape: Ellipsoid point with uncertainty circle
     */
    public static final byte GAD_SHAPES_ELLIPSOID_POINT_WITH_UNCERTAINTY_CIRCLE = (byte)0x02 ;
    /**
     * GAD shape: Ellipsoid point with uncertainty ellipse
     */
    public static final byte GAD_SHAPES_ELLIPSOID_POINT_WITH_UNCERTAINTY_ELLIPSE = (byte)0x04 ;
    /**
     * GAD shape: Ellipsoid point with altitude
     */
    public static final byte GAD_SHAPES_ELLIPSOID_POINT_WITH_ALTITUDE = (byte)0x08 ;
    /**
     * GAD shape: Polygon
     */
    public static final byte GAD_SHAPES_POLYGON = (byte)0x10 ;
    /**
     * GAD shape: Ellipsoid point with altitude and uncertainty ellipsoid
     */
    public static final byte GAD_SHAPES_ELLIPSOID_POINT_WITH_ALTITUDE_AND_UNCERTAINTY_ELLIPSOID = (byte)0x20 ;
    /**
     * GAD shape: Ellipsoid arc
     */
    public static final byte GAD_SHAPES_ELLIPSOID_ARC = (byte)0x40 ;
    /**
     * Preferred NMEA sentence: $_RMC
     */
    public static final byte PREFERRED_NMEA_SENTENCE_RMC = (byte)0x01 ;
    /**
     * Preferred NMEA sentence: $_GGA
     */
    public static final byte PREFERRED_NMEA_SENTENCE_GGA = (byte)0x02 ;
    /**
     * Preferred NMEA sentence: $_GLL
     */
    public static final byte PREFERRED_NMEA_SENTENCE_GLL = (byte)0x04 ;
    /**
     * Preferred NMEA sentence: $_GNS
     */
    public static final byte PREFERRED_NMEA_SENTENCE_GNS = (byte)0x08 ;

    /**
     * Set alpha identifier parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param alphaIdentifier Alpha identifier to set in geographical location request.
     */
    public void setAlphaIdentifier(byte[] alphaIdentifier) ;

    /**
     * Set icon identifier parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param iconIdentifier Icon identifier to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setIconIdentifier(short iconIdentifier) ;

    /**
     * Set horizontal accuracy parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param horizontalAccuracy Horizontal Accuracy to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setHorizontalAccuracy(byte horizontalAccuracy) ;

    /**
     * Set vertical coordinate parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param verticalCoordinate Vertical Coordinate to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setVerticalCoordinate(byte verticalCoordinate) ;

    /**
     * Set velocity parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param velocity Velocity to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setVelocity(byte velocity) ;

    /**
     * Set accepted GAD shapes parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param acceptedGADShapes Preferred GAD Shape(s) to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setAcceptedGADShapes(byte acceptedGADShapes) ;

    /**
     * Set Preferred NMEA sentences parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param preferredNMEASentences Preferred NMEA Sentence(s) to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setPreferredNMEASentences(byte preferredNMEASentences) ;

    /**
     * Set Preferred Maximum Response Time parameter for this object as defined in 3GPP TS 31.111 [7].
     * @param preferredMaximumResponseTime Preferred Max Response Time to set in geographical location request.
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>INCORRECT_PARAMETERS</code> in case of bad parameter is set.
     */
    public void setPreferredMaximumResponseTime(byte preferredMaximumResponseTime) ;

    /**
     * Request a geographical location operation, using parameters defined in the object.
     * A callback mechanism is used to retrieve the result, when available, using
     * the <code>geoLocationEvent</code> object. <br>
     * If the ME has indicated in its Terminal Profile that it supports "class n"
     * ("Geographical Location Request" see TS 31.111[7] for more details) the system will first send out
     * a "Geographical Location Request" proactive command.<br>
     * <ul>
     *    <li>If the Terminal Responds from the ME indicates that the commands can be performed successfully the
     *        actual Geo Location information will be made available through a <code>GeoLocationInformation</code> object.
     *    </li>
     * </ul>
     * @param geoLocationEvent the callback object to be invoked once the result is available.
     *
     * @exception  NullPointerException if <code> geoLocationEvent </code>
     * is <code>null</code>
     *
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>FUNCTION_DISABLED</code> if the ME does not support "class n."
     *      <li><code>UNABLE_TO_PROCESS_COMMAND</code> if Terminal Response from the ME indicates that it cannot perform this command.
     *      <li><code>LOCATION_REQUEST_PROCESSING</code> if there is an on-going location request
     *          (i.e. after sending of Geographical Location Request and before reception of Geographical Location Reporting)
     *      <li><code>HANDLER_NOT_AVAILABLE</code> if there is on-going proactive session
     * </ul>
     */
    public void performGeoLocationRequest(GeoLocationEventListener geoLocationEventListener) throws GeoLocationException ;
}

