package uicc.usim.geolocation ;

/**
 * An object implementing this interface gives access to the Geo Location information
 * received from the ME.
 */
public interface GeoLocationInfo {

    /**
     * Gives the Latitude of the geographical location. <br>
     * When GAD shape is Polygon, the returned value is the Centroid of the Polygon which is the average point of all the latitude and longitude samples.<br>
         * When ME answer with NMEA format: <ul>
         * <li>the Latitude shall be returned if present in the NMEA sentence.
         * <li>only the sentences listed in TS 31.111[7] (i.e. $--RMC, $--GGA, $--GLL, $--GNS) could be used.
         * <li>If two of these sentences are provided in geolocation reporting envelope,only the first NMEA sentence given in the
         * envelope will used to retrieve the Latitude.
         * <li>If none of these sentences are provided in geolocation reporting envelope, exception is thrown.</ul>
     * @return the Latitude according to TS 23.032 [14] <br>
     * A negative value indicates South
     *
     * @exception  GeoLocationException with the following reason codes: <ul>
     *      <li><code>UNABLE_TO_PROCESS_COMMAND</code> if ME does not support "class n" or had issued an error to "Geographical Location Request" proactive
     * command.
     *      <li><code>INCORRECT_INFORMATION</code> if empty data was returned in Geographical Location reporting envelope (i.e. Geographical Location reporting
     * Tag, Length, and Device Identities only), or only none listed in TS 31.111[7] NMEA format is provided.
     * </ul>
     */
    public int getLatitude() ;

    /**
     * Gives the Longitude of the geographical location. <br>
     * When GAD shape is Polygon, the returned value is the Centroid of the Polygon which is the average point of all the
     * latitude and longitude samples. <br>
        * When ME answer with NMEA format: <ul>
        * <li>the Longitude shall be returned if present in the NMEA sentence.
        * <li>only the sentences listed in TS 31.111[7] (i.e. $--RMC, $--GGA, $--GLL, $--GNS) could be used.
        * <li>If two of these sentences are provided in geolocation reporting envelope,only the first NMEA sentence given in
        * the envelope will used to retrieve the Longitude.
        * <li>If none of these sentences are provided in geolocation reporting envelope, exception is thrown.</ul>
     * @return the Longitude according to TS 23.032 [14]
     *
     * @exception  GeoLocationException with the following reason codes: <ul>
        *      <li><code>UNABLE_TO_PROCESS_COMMAND</code> if ME does not support "class n" or had issued an error to "Geographical Location Request"
        * proactive command.
        *      <li><code>INCORRECT_INFORMATION</code> if empty data was returned in Geographical Location reporting envelope (i.e. Geographical Location reporting
        * Tag, Length, and Device Identities only), or only none listed in TS 31.111[7] NMEA format is provided.
        * </ul>
     */
    public int getLongitude() ;

    /**
     * Gives the Altitude of the geographical location. <br>
        * When ME answer with NMEA format: <ul>
        * <li>the Altitude shall be returned if present in the NMEA sentence.
        * <li>only the sentences listed in TS 31.111[7] (i.e. $--RMC, $--GGA, $--GLL, $--GNS) could be used.
        * <li>If two of these sentences are provided in geolocation reporting envelope,only the first NMEA sentence given in
        * the envelope will used to retrieve the Altitude.
        * <li>If none of these sentences are provided in geolocation reporting envelope, exception is thrown.</ul>
     * @return the Altitude according to TS 23.032 [14]
     *
     * @exception  GeoLocationException with the following reason codes: <ul>
        *      <li><code>UNABLE_TO_PROCESS_COMMAND</code> if the ME does not support "class n" or had issued an error to "Geographical Location Request"
        * proactive command.
        *      <li><code>INCORRECT_INFORMATION</code> if empty data was returned in Geographical Location reporting envelope (i.e. Geographical Location reporting
        * Tag, Length, and Device Identities only), or only none listed in TS 31.111[7] NMEA format is provided.
        * </ul>
     */
    public int getAltitude() ;

    /**
     * Gives the Velocity of the geographical location. <br>
     * @param bGetVelocityField Velocity Field to return from the Velocity Information.
     * @return the Velocity according to TS 23.032 [14], -1 if no velocity is given in the
     *         geo location information.
     *
     * @exception  GeoLocationException with the following reason codes: <ul>
        *      <li><code>UNABLE_TO_PROCESS_COMMAND</code> if the ME does not support "class n" or had issued an error to "Geographical Location
        * Request" proactive command.
        *      <li><code>INCORRECT_INFORMATION</code> if empty data was returned in Geographical Location reporting envelope (i.e. Geographical Location reporting
        * Tag, Length, and Device Identities only), or only none listed in TS 31.111[7] NMEA format is provided.
        *      <li><code>INCORRECT_PARAMETERS</code> if incorrect parameters is set.
        * </ul>
     */
    public int getVelocity(byte bGetVelocityField) ;

    /**
     * The byte buffer is formatted either using GAD shapes as defined in TS 23.032 [14] or NMEA format as defined in IEC 61162-1
     *  depending on the format answered by the ME. <br>
     * Returns empty buffer (i.e.: length = 0), when empty data was returned in Geographical Location reporting envelope (i.e.
     * Geographical Location Reporting Tag, Length, and Device Identities only). <br>
     * If the ME answer with NMEA format, only the sentences listed in TS 31.111[7] (i.e. $--RMC, $--GGA, $--GLL, $--GNS) could
     * be copied in the buffer. <br>
     * If two of these sentences are provided in geolocation reporting envelope,only the first NMEA sentence given in the
     * envelope will be copied in the buffer. <br>
     * If none of these sentences are provided in geolocation reporting envelope, empty buffer (i.e.: length = 0) is returned. <p>
     * @param buffer byte array where response of the operation shall be copied.
     * @param offset offset of the result in <code>buffer</code>
     * @param length length of data in <code>buffer</code>
     * @return actual length of the data copied into the <code>buffer</code>
     */
    public short getResponseBuffer(byte[] buffer, short bOffset, short length) ;
}

